from __future__ import annotations

from typing import (
    TYPE_CHECKING, Any, Dict, Generic, List, Mapping, Optional, Tuple, TypeVar,
    Union
)

from typing_extensions import Literal, NotRequired, TypeAlias, TypedDict, final

if TYPE_CHECKING:
    # Flake8 complains about some of these imports where they are only used
    # inside of strings (not annotations) for the type aliases, so we have to
    # disable the check.
    from .interactions import (  # noqa: F401
        ApplicationCommandPermissionsData, InteractionData
    )
    from .reference import Snowflake
    from .resources import (  # noqa: F401
        ApplicationData, AutoModerationActionData, AutoModerationRuleData,
        AutoModerationTriggerTypes, CategoryChannelData, EmojiData, GuildData,
        GuildFeaturesData, GuildMemberData, GuildScheduledEventData,
        MessageData, NewsChannelData, RoleData, StageInstanceData, StickerData,
        TextChannelData, ThreadChannelData, ThreadMemberData, ThreadMetadata,
        UnavailableGuildData, UserData, VoiceChannelData, VoiceStateData,
        WelcomeScreenData
    )

__all__ = (
    'HeartbeatACKEvent', 'IdentifyData', 'IdentifyConnectionProperties', 'IdentifyCommand',
    'ResumeData', 'ResumeCommand', 'HeartbeatCommand', 'RequestGuildMembersData',
    'RequestGuildMembersCommand', 'UpdateVoiceStateData', 'UpdateVoiceStateCommand',
    'PartialActivityData', 'UpdatePresenceData', 'UpdatePresenceCommand', 'HelloData',
    'HelloEvent', 'PartialApplicationData', 'ReadyData', 'ReadyEvent', 'ResumedData',
    'ResumedEvent', 'ReconnectEvent', 'InvalidSessionEvent',
    'ApplicationCommandPermissionsUpdateData', 'ApplicationCommandPermissionsUpdateEvent',
    'ChannelCreateData', 'ChannelCreateEvent', 'ChannelUpdateData', 'ChannelUpdateEvent',
    'ChannelDeleteData', 'ChannelDeleteEvent', 'ThreadCreateData', 'ThreadCreateEvent',
    'ThreadUpdateData', 'ThreadUpdateEvent', 'ThreadDeleteData', 'ThreadDeleteEvent',
    'ThreadListSyncData', 'ThreadListSyncEvent', 'ThreadMemberUpdateData',
    'ThreadMemberUpdateEvent', 'ThreadMembersUpdateData', 'ThreadMembersUpdateEvent',
    'ChannelPinsUpdateData', 'ChannelPinsUpdateEvent', 'GuildCreateData', 'GuildCreateEvent',
    'GuildUpdateData', 'GuildUpdateEvent', 'GuildDeleteData', 'GuildDeleteEvent',
    'GuildBanAddData', 'GuildBanAddEvent', 'GuildBanRemoveData', 'GuildBanRemoveEvent',
    'GuildEmojisUpdateData', 'GuildEmojisUpdateEvent', 'GuildStickersUpdateData',
    'GuildStickersUpdateEvent', 'GuildIntergrationsUpdateData',
    'GuildIntergrationsUpdateEvent', 'GuildMemberAddData', 'GuildMemberAddEvent',
    'GuildMemberRemoveData', 'GuildMemberRemoveEvent', 'GuildMemberUpdateData',
    'GuildMemberUpdateEvent', 'GuildMembersChunkData', 'GuildMembersChunkEvent',
    'GuildRoleCreateData', 'GuildRoleCreateEvent', 'GuildRoleCreateEvent',
    'GuildRoleUpdateData', 'GuildRoleUpdateEvent', 'GuildRoleDeleteData',
    'GuildRoleDeleteEvent', 'GuildScheduledEventCreateData', 'GuildScheduledEventCreateEvent',
    'GuildScheduledEventUpdateData', 'GuildScheduledEventUpdateEvent',
    'GuildScheduledEventDeleteData', 'GuildScheduledEventDeleteEvent',
    'GuildScheduledEventUserAddData', 'GuildScheduledEventUserAddEvent',
    'GuildScheduledEventUserRemoveData', 'GuildScheduledEventUserRemoveEvent',
    'InviteCreateData', 'InviteCreateEvent', 'InviteDeleteData', 'InviteDeleteEvent',
    'MessageCreateData', 'MessageCreateEvent', 'MessageUpdateData', 'MessageUpdateEvent',
    'MessageDeleteData', 'MessageDeleteEvent', 'MessageDeleteBulkData',
    'MessageDeleteBulkEvent', 'MessageReactionAddData', 'MessageReactionAddEvent',
    'MessageReactionRemoveData', 'MessageReactionRemoveEvent', 'MessageReactionRemoveAllData',
    'MessageReactionRemoveAllEvent', 'MessageReactionRemoveEmojiData',
    'MessageReactionRemoveEmojiEvent', 'PresenceUpdateData', 'ClientStatusData',
    'ActivityData', 'ActivityTimestampsData', 'ActivityEmojiData', 'ActivityPartyData',
    'ActivityAssetsData', 'ActivitySecretsData', 'ActivityButtonData', 'TypingStartData',
    'TypingStartEvent', 'VoiceStateUpdateData', 'VoiceStateUpdateEvent',
    'VoiceServerUpdateData', 'VoiceServerUpdateEvent', 'WebhooksUpdateData',
    'WebhooksUpdateEvent', 'InteractionCreateData', 'InteractionCreateEvent',
    'StageInstanceCreateData', 'StageInstanceCreateEvent', 'StageInstanceUpdateData',
    'StageInstanceUpdateEvent', 'StageInstanceDeleteData', 'StageInstanceDeleteEvent',
    'GetGatewayData', 'GetGatewayBotData', 'SessionStartLimitData', 'DispatchEvent',
    'GatewayEvent'
)


_D = TypeVar('_D', bound='Mapping[str, Any]')
_T = TypeVar('_T', bound='str')  # Literal


@final
class GenericDispatchEvent(TypedDict, Generic[_T, _D]):
    """Generic Discord dispatch event.

    This is a generic TypedDict for creating typings for dispatch events. The
    resulting TypedDict contains the `op`, `d`, `s` and `t` fields. To get the
    typing for the inner `d` key, grab the second argument to the generic.

    The reason this doesn't have a leading underscore is to look nicer when
    previewed in editors and IDEs but the actual `GenericDispatchEvent()`
    should be considered private - use the public, typed, versions.
    """
    op: Literal[0]
    d: _D
    s: int
    t: _T


# https://discord.com/developers/docs/topics/gateway#heartbeating-example-gateway-heartbeat-ack


@final
class HeartbeatACKEvent(TypedDict):
    op: Literal[11]


# https://discord.com/developers/docs/topics/gateway#identify


@final
class IdentifyData(TypedDict):
    token: str
    properties: IdentifyConnectionProperties
    compress: NotRequired[bool]
    large_threshold: NotRequired[int]
    shard: NotRequired[Union[Tuple[int, int], List[int]]]
    presence: NotRequired[UpdatePresenceData]
    intents: int


@final
class IdentifyConnectionProperties(TypedDict):
    os: str
    browser: str
    device: str


@final
class IdentifyCommand(TypedDict):
    op: Literal[2]
    d: IdentifyData


# https://discord.com/developers/docs/topics/gateway#resume


@final
class ResumeData(TypedDict):
    token: str
    session_id: str
    seq: int


@final
class ResumeCommand(TypedDict):
    op: Literal[6]
    d: ResumeData


# https://discord.com/developers/docs/topics/gateway#heartbeat


@final
class HeartbeatCommand(TypedDict):
    op: Literal[1]
    d: Optional[int]


# https://discord.com/developers/docs/topics/gateway#request-guild-members


@final
class _QueryRequestMembersCommand(TypedDict):
    guild_id: Snowflake
    query: str
    limit: int
    presences: NotRequired[bool]
    user_ids: NotRequired[Union[Snowflake, List[Snowflake]]]
    nonce: NotRequired[str]


@final
class _UserIDsRequestMembersCommand(TypedDict):
    guild_id: Snowflake
    presences: NotRequired[bool]
    user_ids: Union[Snowflake, List[Snowflake]]
    nonce: NotRequired[str]


# This enforces the fact that 'limit' is required when 'query' is set.
RequestGuildMembersData = Union[_QueryRequestMembersCommand, _UserIDsRequestMembersCommand]


@final
class RequestGuildMembersCommand(TypedDict):
    op: Literal[8]
    d: RequestGuildMembersData


# https://discord.com/developers/docs/topics/gateway#update-voice-state


@final
class UpdateVoiceStateData(TypedDict):
    guild_id: Snowflake
    channel_id: Optional[Snowflake]
    self_mute: bool
    self_deaf: bool


@final
class UpdateVoiceStateCommand(TypedDict):
    op: Literal[4]
    d: UpdateVoiceStateData


# https://discord.com/developers/docs/topics/gateway#update-presence


@final
class PartialActivityData(TypedDict):
    name: str
    type: Literal[0, 1, 2, 3, 4, 5]
    url: NotRequired[str]


@final
class UpdatePresenceData(TypedDict):
    since: Optional[int]
    activities: List[PartialActivityData]
    status: Literal['online', 'idle', 'dnd', 'invisible']
    afk: bool


@final
class UpdatePresenceCommand(TypedDict):
    op: Literal[3]
    d: UpdatePresenceData


# https://discord.com/developers/docs/topics/gateway#hello


@final
class HelloData(TypedDict):
    heartbeat_interval: int


@final
class HelloEvent(TypedDict):
    op: Literal[10]
    d: HelloData
    s: None
    t: None


# https://discord.com/developers/docs/topics/gateway#ready


@final
class PartialApplicationData(TypedDict):
    id: Snowflake
    flags: int


@final
class ReadyData(TypedDict):
    v: int
    user: UserData
    guilds: List[UnavailableGuildData]
    session_id: str
    shard: NotRequired[Union[Tuple[int, int], List[int]]]
    application: PartialApplicationData


ReadyEvent = GenericDispatchEvent[Literal['READY'], ReadyData]


# https://discord.com/developers/docs/topics/gateway#resumed


@final
class ResumedData(TypedDict):
    ...  # It only has an undocumented _trace field


ResumedEvent = GenericDispatchEvent[Literal['RESUMED'], ResumedData]


# https://discord.com/developers/docs/topics/gateway#reconnect


@final
class ReconnectEvent(TypedDict):
    op: Literal[7]
    d: None
    s: None
    t: None


# https://discord.com/developers/docs/topics/gateway#invalid-session


@final
class InvalidSessionEvent(TypedDict):
    op: Literal[9]
    d: bool
    s: None
    t: None

# https://discord.com/developers/docs/topics/gateway#get-gateway-example-response


@final
class GetGatewayData(TypedDict):
    url: str

# https://discord.com/developers/docs/topics/gateway#application-command-permissions-update


ApplicationCommandPermissionsUpdateData: TypeAlias = 'ApplicationCommandPermissionsData'


ApplicationCommandPermissionsUpdateEvent = GenericDispatchEvent[
    Literal['APPLICATION_COMMAND_PERMISSIONS_UPDATE'],
    ApplicationCommandPermissionsUpdateData
]


# https://discord.com/developers/docs/topics/gateway#auto-moderation-rule-create


AutoModerationRuleCreateData: TypeAlias = 'AutoModerationRuleData'


AutoModerationRuleCreateEvent = GenericDispatchEvent[
    Literal['AUTO_MODERATION_RULE_CREATE'],
    AutoModerationRuleCreateData
]


# https://discord.com/developers/docs/topics/gateway#auto-moderation-rule-update


AutoModerationRuleUpdateData: TypeAlias = 'AutoModerationRuleData'


AutoModerationRuleUpdateEvent = GenericDispatchEvent[
    Literal['AUTO_MODERATION_RULE_UPDATE'],
    AutoModerationRuleUpdateData
]


# https://discord.com/developers/docs/topics/gateway#auto-moderation-rule-delete


AutoModerationRuleDeleteData: TypeAlias = 'AutoModerationRuleData'


AutoModerationRuleDeleteEvent = GenericDispatchEvent[
    Literal['AUTO_MODERATION_RULE_DELETE'],
    AutoModerationRuleDeleteData
]


# https://discord.com/developers/docs/topics/gateway#auto-moderation-action-execution


@final
class AutoModerationActionExecutionData(TypedDict):
    guild_id: Snowflake
    action: AutoModerationActionData
    rule_id: Snowflake
    rule_trigger_type: AutoModerationTriggerTypes
    user_id: Snowflake
    channel_id: NotRequired[Snowflake]
    message_id: NotRequired[Snowflake]
    alert_system_message_id: NotRequired[Snowflake]
    content: str
    matched_keyword: Optional[str]
    matched_content: Optional[str]


AutoModerationActionExecutionEvent = GenericDispatchEvent[
    Literal['AUTO_MODERATION_ACTION_EXECUTION'],
    AutoModerationActionExecutionData
]


# https://discord.com/developers/docs/topics/gateway#channels


# Utility for the events below
_GuildChannelData = Union[
    'TextChannelData', 'NewsChannelData', 'VoiceChannelData', 'CategoryChannelData'
]


# https://discord.com/developers/docs/topics/gateway#channel-create


ChannelCreateData = _GuildChannelData
ChannelCreateEvent = GenericDispatchEvent[Literal['CHANNEL_CREATE'], ChannelCreateData]


# https://discord.com/developers/docs/topics/gateway#channel-update


ChannelUpdateData = _GuildChannelData
ChannelUpdateEvent = GenericDispatchEvent[Literal['CHANNEL_UPDATE'], ChannelUpdateData]


# https://discord.com/developers/docs/topics/gateway#channel-delete


ChannelDeleteData = _GuildChannelData
ChannelDeleteEvent = GenericDispatchEvent[Literal['CHANNEl_DELETE'], ChannelDeleteData]


# https://discord.com/developers/docs/topics/gateway#thread-create


@final
class ThreadCreateData(TypedDict):
    id: Snowflake
    type: Literal[10, 11, 12]
    guild_id: NotRequired[Snowflake]
    name: str
    last_message_id: Optional[Snowflake]
    rate_limit_per_user: int
    owner_id: Snowflake
    parent_id: Optional[Snowflake]
    last_pin_timestamp: NotRequired[Optional[str]]
    message_count: int
    member_count: int
    thread_metadata: ThreadMetadata
    member: NotRequired[ThreadMemberData]

    newly_created: bool  # Extra THREAD_CREATE field


ThreadCreateEvent = GenericDispatchEvent[Literal['THREAD_CREATE'], ThreadCreateData]


# https://discord.com/developers/docs/topics/gateway#thread-update


ThreadUpdateData: TypeAlias = 'ThreadChannelData'
ThreadUpdateEvent = GenericDispatchEvent[Literal['THREAD_UPDATE'], ThreadUpdateData]


# https://discord.com/developers/docs/topics/gateway#thread-delete


@final
class ThreadDeleteData(TypedDict):
    id: Snowflake
    guild_id: Snowflake
    parent_id: Snowflake
    type: Literal[10, 11, 12]


ThreadDeleteEvent = GenericDispatchEvent[Literal['THREAD_DELETE'], ThreadDeleteData]


# https://discord.com/developers/docs/topics/gateway#thread-list-sync


@final
class ThreadListSyncData(TypedDict):
    guild_id: Snowflake
    channel_ids: NotRequired[List[Snowflake]]
    threads: List[ThreadChannelData]
    members: List[ThreadMemberData]


ThreadListSyncEvent = GenericDispatchEvent[Literal['THREAD_LIST_SYNC'], ThreadListSyncData]


# https://discord.com/developers/docs/topics/gateway#thread-member-update


@final
class ThreadMemberUpdateData(TypedDict):
    # 'id' and 'user_id' is only missing in the GUILD_CREATE event, so we know
    # that they are present in this event (hence the missing NotRequired
    # compared to ThreadMemberData).
    id: Snowflake
    user_id: Snowflake
    guild_id: Snowflake
    join_timestamp: str
    flags: int


ThreadMemberUpdateEvent = GenericDispatchEvent[
    Literal['THREAD_MEMBER_UPDATE'], ThreadMemberUpdateData
]


# https://discord.com/developers/docs/topics/gateway#thread-members-update


@final
class ThreadMembersUpdateData(TypedDict):
    id: Snowflake
    guild_id: Snowflake
    member_count: int
    added_members: NotRequired[List[ThreadMemberData]]
    removed_member_ids: NotRequired[List[Snowflake]]


ThreadMembersUpdateEvent = GenericDispatchEvent[
    Literal['THREAD_MEMBERS_UPDATE'], ThreadMembersUpdateData
]


# https://discord.com/developers/docs/topics/gateway#channel-pins-update


@final
class ChannelPinsUpdateData(TypedDict):
    guild_id: NotRequired[Snowflake]
    channel_id: Snowflake
    last_pin_timestamp: NotRequired[Optional[str]]


ChannelPinsUpdateEvent = GenericDispatchEvent[
    Literal['CHANNEL_PINS_UPDATE'], ChannelPinsUpdateData
]


# https://discord.com/developers/docs/topics/gateway#guild-create


@final
class GuildCreateData(TypedDict):
    id: str
    name: str
    icon: Optional[str]
    icon_hash: NotRequired[Optional[str]]
    splash: Optional[str]
    discovery_splash: Optional[str]
    owner: NotRequired[bool]
    owner_id: str
    permissions: NotRequired[str]
    afk_channel_id: Optional[str]
    afk_timeout: int
    widget_enabled: NotRequired[bool]
    widget_channel_id: NotRequired[Optional[str]]
    verification_level: Literal[0, 1, 2, 3, 4]
    default_message_notifications: Literal[0, 1]
    explicit_content_filter: Literal[0, 1, 2]
    roles: List[RoleData]
    emojis: List[EmojiData]
    features: List[GuildFeaturesData]
    mfa_level: Literal[0, 1]
    application_id: Optional[str]
    system_channel_id: Optional[str]
    system_channel_flags: int
    rules_channel_id: Optional[str]
    max_presences: NotRequired[Optional[int]]
    max_members: NotRequired[int]
    vanity_url_code: Optional[str]
    description: Optional[str]
    banner: Optional[str]
    premium_tier: Literal[0, 1, 2, 3]
    premium_subscription_count: NotRequired[int]
    preferred_locale: str
    public_updates_channel_id: Optional[str]
    max_video_channel_users: NotRequired[int]
    approximate_member_count: NotRequired[int]
    approximate_presence_count: NotRequired[int]
    welcome_screen: NotRequired[WelcomeScreenData]
    nsfw_level: Literal[0, 1, 2, 3]
    stickers: NotRequired[List[StickerData]]
    premium_progress_bar_enabled: bool

    # Extra GUILD_CREATE fields
    joined_at: str
    large: bool
    unavailable: bool
    member_count: int
    voice_states: List[VoiceStateData]
    members: List[GuildMemberData]
    channels: List[_GuildChannelData]
    threads: List[ThreadChannelData]
    presences: List[UpdatePresenceData]
    stage_instances: List[StageInstanceData]
    guild_scheduled_events: List[GuildScheduledEventData]


GuildCreateEvent = GenericDispatchEvent[Literal['GUILD_CREATE'], GuildCreateData]


# https://discord.com/developers/docs/topics/gateway#guild-update


GuildUpdateData: TypeAlias = 'GuildData'
GuildUpdateEvent = GenericDispatchEvent[Literal['GUILD_UPDATE'], GuildUpdateData]


# https://discord.com/developers/docs/topics/gateway#guild-delete


GuildDeleteData: TypeAlias = 'UnavailableGuildData'
GuildDeleteEvent = GenericDispatchEvent[Literal['GUILD_DELETE'], 'GuildDeleteData']


# https://discord.com/developers/docs/topics/gateway#guild-ban-add


@final
class GuildBanAddData(TypedDict):
    guild_id: Snowflake
    user: UserData


GuildBanAddEvent = GenericDispatchEvent[Literal['GUILD_BAN_ADD'], GuildBanAddData]


# https://discord.com/developers/docs/topics/gateway#guild-ban-remove


@final
class GuildBanRemoveData(TypedDict):
    guild_id: Snowflake
    user: UserData


GuildBanRemoveEvent = GenericDispatchEvent[Literal['GUILD_BAN_REMOVE'], GuildBanRemoveData]


# https://discord.com/developers/docs/topics/gateway#guild-emojis-update


@final
class GuildEmojisUpdateData(TypedDict):
    guild_id: Snowflake
    emojis: List[EmojiData]


GuildEmojisUpdateEvent = GenericDispatchEvent[
    Literal['GUILD_EMOJIS_UPDATE'], GuildEmojisUpdateData
]


# https://discord.com/developers/docs/topics/gateway#guild-stickers-update


@final
class GuildStickersUpdateData(TypedDict):
    guild_id: Snowflake
    stickers: List[StickerData]


GuildStickersUpdateEvent = GenericDispatchEvent[
    Literal['GUILD_STICKERS_UPDATE'], GuildStickersUpdateData
]


# https://discord.com/developers/docs/topics/gateway#guild-integrations-update


@final
class GuildIntergrationsUpdateData(TypedDict):
    guild_id: Snowflake


GuildIntergrationsUpdateEvent = GenericDispatchEvent[
    Literal['GUILD_INTEGRATIONS_UPDATE'], GuildIntergrationsUpdateData
]


# https://discord.com/developers/docs/topics/gateway#guild-member-add


@final
class GuildMemberAddData(TypedDict):
    user: UserData  # Always present in GUILD_MEMBER_ADD events
    nick: NotRequired[Optional[str]]
    avatar: NotRequired[Optional[str]]
    roles: List[Snowflake]
    joined_at: str
    premium_since: NotRequired[Optional[str]]
    deaf: bool
    mute: bool
    pending: NotRequired[bool]
    permissions: NotRequired[str]

    guild_id: Snowflake  # Extra field


GuildMemberAddEvent = GenericDispatchEvent[Literal['GUILD_MEMBER_ADD'], GuildMemberAddData]


# https://discord.com/developers/docs/topics/gateway#guild-member-remove


@final
class GuildMemberRemoveData(TypedDict):
    user: UserData
    guild_id: Snowflake


GuildMemberRemoveEvent = GenericDispatchEvent[
    Literal['GUILD_MEMBER_REMOVE'], GuildMemberRemoveData
]


# https://discord.com/developers/docs/topics/gateway#guild-member-update


@final
class GuildMemberUpdateData(TypedDict):
    guild_id: Snowflake
    roles: List[Snowflake]
    user: List[UserData]
    nick: NotRequired[Optional[str]]
    avatar: Optional[str]
    joined_at: Optional[str]
    premium_since: NotRequired[Optional[str]]
    deaf: NotRequired[bool]
    mute: NotRequired[bool]
    pending: NotRequired[bool]
    communication_disabled_until: NotRequired[Optional[str]]


GuildMemberUpdateEvent = GenericDispatchEvent[
    Literal['GUILD_MEMBER_UPDATE'], GuildMemberUpdateData
]


# https://discord.com/developers/docs/topics/gateway#guild-members-chunk


@final
class GuildMembersChunkData(TypedDict):
    guild_id: Snowflake
    members: List[GuildMemberData]
    chunk_index: int
    chunk_count: int
    not_found: NotRequired[List[Snowflake]]
    presences: NotRequired[List[PresenceUpdateData]]
    nonce: NotRequired[str]


GuildMembersChunkEvent = GenericDispatchEvent[
    Literal['GUILD_MEMBERS_CHUNK'], GuildMembersChunkData
]


# https://discord.com/developers/docs/topics/gateway#guild-role-create


@final
class GuildRoleCreateData(TypedDict):
    guild_id: Snowflake
    role: RoleData


GuildRoleCreateEvent = GenericDispatchEvent[Literal['GUILD_ROLE_CREATE'], GuildRoleCreateData]


# https://discord.com/developers/docs/topics/gateway#guild-role-update


@final
class GuildRoleUpdateData(TypedDict):
    guild_id: Snowflake
    role: RoleData


GuildRoleUpdateEvent = GenericDispatchEvent[Literal['GUILD_ROLE_UPDATE'], GuildRoleUpdateData]


# https://discord.com/developers/docs/topics/gateway#guild-role-delete


@final
class GuildRoleDeleteData(TypedDict):
    guild_id: Snowflake
    role_id: Snowflake


GuildRoleDeleteEvent = GenericDispatchEvent[Literal['GUILD_ROLE_DELETE'], GuildRoleDeleteData]


# https://discord.com/developers/docs/topics/gateway#guild-scheduled-event-create


GuildScheduledEventCreateData: TypeAlias = 'GuildScheduledEventData'
GuildScheduledEventCreateEvent = GenericDispatchEvent[
    Literal['GUILD_SCHEDULED_EVENT_CREATE'], GuildScheduledEventCreateData
]


# https://discord.com/developers/docs/topics/gateway#guild-scheduled-event-update


GuildScheduledEventUpdateData: TypeAlias = 'GuildScheduledEventData'
GuildScheduledEventUpdateEvent = GenericDispatchEvent[
    Literal['GUILD_SCHEDULED_EVENT_UPDATE'], GuildScheduledEventUpdateData
]


# https://discord.com/developers/docs/topics/gateway#guild-scheduled-event-delete


GuildScheduledEventDeleteData: TypeAlias = 'GuildScheduledEventData'
GuildScheduledEventDeleteEvent = GenericDispatchEvent[
    Literal['GUILD_SCHEDULED_EVENT_DELETE'], GuildScheduledEventDeleteData
]


# https://discord.com/developers/docs/topics/gateway#guild-scheduled-event-user-add


@final
class GuildScheduledEventUserAddData(TypedDict):
    guild_scheduled_event_id: Snowflake
    user_id: Snowflake
    guild_id: Snowflake


GuildScheduledEventUserAddEvent = GenericDispatchEvent[
    Literal['GUILD_SCHEDULED_EVENT_USER_ADD'], GuildScheduledEventUserAddData
]


# https://discord.com/developers/docs/topics/gateway#guild-scheduled-event-user-remove


@final
class GuildScheduledEventUserRemoveData(TypedDict):
    guild_scheduled_event_id: Snowflake
    user_id: Snowflake
    guild_id: Snowflake


GuildScheduledEventUserRemoveEvent = GenericDispatchEvent[
    Literal['GUILD_SCHEDULED_EVENT_USER_REMOVE'], GuildScheduledEventUserRemoveData
]


# https://discord.com/developers/docs/topics/gateway#invite-create


@final
class InviteCreateData(TypedDict):
    channel_id: Snowflake
    code: str
    created_at: str
    guild_id: NotRequired[Snowflake]
    inviter: NotRequired[UserData]
    max_age: int
    max_uses: int
    target_type: NotRequired[Literal[1, 2]]
    target_user: NotRequired[UserData]
    target_application: NotRequired[ApplicationData]
    temporary: bool
    uses: int


InviteCreateEvent = GenericDispatchEvent[Literal['INVITE_CREATE'], InviteCreateData]


# https://discord.com/developers/docs/topics/gateway#invite-delete


@final
class InviteDeleteData(TypedDict):
    channel_id: Snowflake
    guild_id: NotRequired[Snowflake]
    code: str


InviteDeleteEvent = GenericDispatchEvent[Literal['INVITE_DELETE'], InviteDeleteData]


# https://discord.com/developers/docs/topics/gateway#message-create


MessageCreateData: TypeAlias = 'MessageData'
MessageCreateEvent = GenericDispatchEvent[Literal['MESSAGE_CREATE'], MessageCreateData]


# https://discord.com/developers/docs/topics/gateway#message-update


MessageUpdateData: TypeAlias = 'MessageData'
MessageUpdateEvent = GenericDispatchEvent[Literal['MESSAGE_UPDATE'], MessageUpdateData]


# https://discord.com/developers/docs/topics/gateway#message-delete


@final
class MessageDeleteData(TypedDict):
    id: Snowflake
    channel_id: Snowflake
    guild_id: NotRequired[Snowflake]


MessageDeleteEvent = GenericDispatchEvent[Literal['MESSAGE_DELETE'], MessageDeleteData]


# https://discord.com/developers/docs/topics/gateway#message-delete-bulk


@final
class MessageDeleteBulkData(TypedDict):
    ids: List[Snowflake]
    channel_id: Snowflake
    guild_id: NotRequired[Snowflake]


MessageDeleteBulkEvent = GenericDispatchEvent[
    Literal['MESSAGE_DELETE_BULK'], MessageDeleteBulkData
]


# https://discord.com/developers/docs/topics/gateway#message-reaction-add


@final
class MessageReactionAddData(TypedDict):
    user_id: Snowflake
    channel_id: Snowflake
    message_id: Snowflake
    guild_id: NotRequired[Snowflake]
    member: NotRequired[GuildMemberData]
    emoji: EmojiData


MessageReactionAddEvent = GenericDispatchEvent[
    Literal['MESSAGE_REACTION_ADD'], MessageReactionAddData
]


# https://discord.com/developers/docs/topics/gateway#message-reaction-remove


@final
class MessageReactionRemoveData(TypedDict):
    user_id: Snowflake
    channel_id: Snowflake
    message_id: Snowflake
    guild_id: NotRequired[Snowflake]
    emoji: EmojiData


MessageReactionRemoveEvent = GenericDispatchEvent[
    Literal['MESSAGE_REACTION_REMOVE'], MessageReactionRemoveData
]


# https://discord.com/developers/docs/topics/gateway#message-reaction-remove-all


@final
class MessageReactionRemoveAllData(TypedDict):
    channel_id: Snowflake
    message_id: Snowflake
    guild_id: NotRequired[Snowflake]


MessageReactionRemoveAllEvent = GenericDispatchEvent[
    Literal['MESSAGE_REACTION_REMOVE_ALL'], MessageReactionRemoveAllData
]


# https://discord.com/developers/docs/topics/gateway#message-reaction-remove-emoji


@final
class MessageReactionRemoveEmojiData(TypedDict):
    channel_id: Snowflake
    guild_id: NotRequired[Snowflake]
    message_id: Snowflake
    emoji: EmojiData


MessageReactionRemoveEmojiEvent = GenericDispatchEvent[
    Literal['MESSAGE_REACTION_REMOVE_EMOJI'], MessageReactionRemoveEmojiData
]


# https://discord.com/developers/docs/topics/gateway#presence-update


@final
class PresenceUpdateData(TypedDict):
    user: UserData
    guild_id: Snowflake
    status: Literal['idle', 'dnd', 'online', 'offline']
    activities: List[ActivityData]
    client_status: ClientStatusData


@final
class ClientStatusData(TypedDict):
    desktop: NotRequired[str]
    mobile: NotRequired[str]
    web: NotRequired[str]


# https://discord.com/developers/docs/topics/gateway#activity-object-activity-structure


@final
class ActivityData(TypedDict):
    name: str
    type: Literal[0, 1, 2, 3, 4, 5]
    url: NotRequired[Optional[str]]
    created_at: int
    timestamps: NotRequired[ActivityTimestampsData]
    application_id: NotRequired[Snowflake]
    details: NotRequired[Optional[str]]
    state: NotRequired[Optional[str]]
    emoji: NotRequired[Optional[ActivityEmojiData]]
    party: NotRequired[Optional[ActivityPartyData]]
    assets: NotRequired[ActivityAssetsData]
    secrets: NotRequired[ActivitySecretsData]
    instance: NotRequired[bool]
    flags: NotRequired[int]
    buttons: NotRequired[List[ActivityButtonData]]


@final
class ActivityTimestampsData(TypedDict):
    start: NotRequired[int]
    end: NotRequired[int]


@final
class ActivityEmojiData(TypedDict):
    name: str
    id: NotRequired[Snowflake]
    animated: NotRequired[bool]


@final
class ActivityPartyData(TypedDict):
    id: NotRequired[str]
    size: NotRequired[Union[Tuple[int, int], List[int]]]


@final
class ActivityAssetsData(TypedDict):
    large_image: NotRequired[str]
    large_text: NotRequired[str]
    small_image: NotRequired[str]
    small_test: NotRequired[str]


@final
class ActivitySecretsData(TypedDict):
    join: NotRequired[str]
    spectate: NotRequired[str]
    match: NotRequired[str]


@final
class ActivityButtonData(TypedDict):
    label: str
    url: str


# https://discord.com/developers/docs/topics/gateway#typing-start


@final
class TypingStartData(TypedDict):
    channel_id: Snowflake
    guild_id: NotRequired[Snowflake]
    user_id: Snowflake
    timestamp: int
    member: NotRequired[GuildMemberData]


TypingStartEvent = GenericDispatchEvent[Literal['TYPING_START'], TypingStartData]


# https://discord.com/developers/docs/topics/gateway#voice-state-update


VoiceStateUpdateData: TypeAlias = 'VoiceStateData'
VoiceStateUpdateEvent = GenericDispatchEvent[
    Literal['VOICE_STATUS_UPDATE'], VoiceStateUpdateData
]


# https://discord.com/developers/docs/topics/gateway#voice-server-update


@final
class VoiceServerUpdateData(TypedDict):
    token: str
    guild_id: Snowflake
    endpoint: Optional[str]


VoiceServerUpdateEvent = GenericDispatchEvent[
    Literal['VOICE_SERVER_UPDATE'], VoiceServerUpdateData
]


# https://discord.com/developers/docs/topics/gateway#webhooks-update


@final
class WebhooksUpdateData(TypedDict):
    guild_id: Snowflake
    channel_id: Snowflake


WebhooksUpdateEvent = GenericDispatchEvent[Literal['WEBHOOKS_UPDATE'], WebhooksUpdateData]


# https://discord.com/developers/docs/topics/gateway#interaction-create


InteractionCreateData: TypeAlias = 'InteractionData'
InteractionCreateEvent = GenericDispatchEvent[
    Literal['INTERACTION_CREATE'], InteractionCreateData
]


# https://discord.com/developers/docs/topics/gateway#stage-instance-create


StageInstanceCreateData: TypeAlias = 'StageInstanceData'
StageInstanceCreateEvent = GenericDispatchEvent[
    Literal['STAGE_INSTANCE_CREATE'], StageInstanceCreateData
]


# https://discord.com/developers/docs/topics/gateway#stage-instance-update


StageInstanceUpdateData: TypeAlias = 'StageInstanceData'
StageInstanceUpdateEvent = GenericDispatchEvent[
    Literal['STAGE_INSTANCE_UPDATE'], StageInstanceUpdateData
]


# https://discord.com/developers/docs/topics/gateway#stage-instance-delete


StageInstanceDeleteData: TypeAlias = 'StageInstanceData'
StageInstanceDeleteEvent = GenericDispatchEvent[
    Literal['STAGE_INSTANCE_DELETE'], StageInstanceDeleteData
]


# https://discord.com/developers/docs/topics/gateway#get-gateway-bot-json-response


@final
class GetGatewayBotData(TypedDict):
    url: str
    shards: int
    session_start_limit: SessionStartLimitData


# https://discord.com/developers/docs/topics/gateway#session-start-limit-object-session-start-limit-structure


@final
class SessionStartLimitData(TypedDict):
    total: int
    remaining: int
    reset_after: int
    max_concurrency: int


# Generalized unions for the typings in this file


DispatchEvent = GenericDispatchEvent[str, Dict[str, Any]]


GatewayEvent = Union[
    HeartbeatACKEvent, HelloEvent, ReadyEvent, ResumedEvent, DispatchEvent,
    ReconnectEvent, InvalidSessionEvent,
]


GatewayCommand = Union[
    IdentifyCommand, ResumeCommand, HeartbeatCommand, RequestGuildMembersCommand,
    UpdateVoiceStateCommand
]



from typing import Dict, List, Union

from typing_extensions import TypedDict, final

__all__ = [
    'Snowflake', 'HTTPErrorData', 'InnerHTTPErrorsData', 'NestedHTTPErrorsData',
    'HTTPErrorResponseData',
]


# Somewhat useless but good to distinguish the difference, perhaps in the
# future Python's typing system will have something similar to TypeScript's
# "Template Literal Types" which is able to describe a snowflake.
# Discord handles integers on endpoints but all snowflakes received by the API
# will be strings.
# https://discord.com/developers/docs/reference#snowflakes
Snowflake = Union[str, int]


# https://discord.com/developers/docs/reference#error-messages


@final
class HTTPErrorData(TypedDict):
    code: str
    message: str


@final
class InnerHTTPErrorsData(TypedDict):
    _errors: List[HTTPErrorData]


NestedHTTPErrorsData = Union[
    Dict[str, 'NestedHTTPErrorsData'],
    InnerHTTPErrorsData
]


@final
class HTTPErrorResponseData(TypedDict):
    code: int
    errors: NestedHTTPErrorsData
    message: str



from __future__ import annotations

from typing import List, Union

from typing_extensions import Literal, TypedDict, final

from ..reference import Snowflake

__all__ = [
    'AutoModerationRuleData', 'AutoModerationTriggerTypes',
    'AutoModerationTriggerMetadataData', 'AutoModerationKeywordPresetTypes',
    'AutoModerationEventTypes', 'AutoModerationActionData', 'AutoModerationActionTypes',

]


# https://discord.com/developers/docs/resources/auto-moderation#auto-moderation-rule-object-auto-moderation-rule-structure


@final
class KeywordAutoModerationRuleData(TypedDict):
    id: Snowflake
    guild_id: Snowflake
    name: str
    creator_id: Snowflake
    event_type: AutoModerationEventTypes
    trigger_type: Literal[1]
    trigger_metadata: KeywordTriggerMetadataData
    actions: List[AutoModerationActionData]
    enabled: bool
    exempt_roles: List[Snowflake]
    exempt_channels: List[Snowflake]


@final
class KeywordPresetAutoModerationRuleData(TypedDict):
    id: Snowflake
    guild_id: Snowflake
    name: str
    creator_id: Snowflake
    event_type: AutoModerationEventTypes
    trigger_type: Literal[4]
    trigger_metadata: KeywordPresetTriggerMetadataData
    actions: List[AutoModerationActionData]
    enabled: bool
    exempt_roles: List[Snowflake]
    exempt_channels: List[Snowflake]


@final
class HarmfulLinkSpamAutoModerationRuleData(TypedDict):
    id: Snowflake
    guild_id: Snowflake
    name: str
    creator_id: Snowflake
    event_type: AutoModerationEventTypes
    trigger_type: Literal[2, 3]
    trigger_metadata: EmptyTriggerMetadataData
    actions: List[AutoModerationActionData]
    enabled: bool
    exempt_roles: List[Snowflake]
    exempt_channels: List[Snowflake]


AutoModerationRuleData = Union[
    KeywordAutoModerationRuleData, KeywordPresetAutoModerationRuleData,
    HarmfulLinkSpamAutoModerationRuleData
]


# https://discord.com/developers/docs/resources/auto-moderation#auto-moderation-rule-object-trigger-types


AutoModerationTriggerTypes = Literal[1, 2, 3, 4]


# https://discord.com/developers/docs/resources/auto-moderation#auto-moderation-rule-object-trigger-metadata


@final
class KeywordTriggerMetadataData(TypedDict):
    keyword_filter: List[str]


@final
class KeywordPresetTriggerMetadataData(TypedDict):
    presets: List[AutoModerationKeywordPresetTypes]


@final
class EmptyTriggerMetadataData(TypedDict):
    ...


AutoModerationTriggerMetadataData = Union[
    KeywordTriggerMetadataData, KeywordPresetTriggerMetadataData,
    EmptyTriggerMetadataData
]


# https://discord.com/developers/docs/resources/auto-moderation#auto-moderation-rule-object-keyword-preset-types


AutoModerationKeywordPresetTypes = Literal[1, 2, 3]


# https://discord.com/developers/docs/resources/auto-moderation#auto-moderation-rule-object-event-types


AutoModerationEventTypes = Literal[1]


# https://discord.com/developers/docs/resources/auto-moderation#auto-moderation-action-object-auto-moderation-action-structure


@final
class BlockMessageAutoModerationActionData(TypedDict):
    type: Literal[1]


@final
class SendAlertMessageAutoModerationActionData(TypedDict):
    type: Literal[2]
    metadata: SendAlertMessageAutoModerationActionMetadataData


@final
class TimeoutAutoModerationActionData(TypedDict):
    type: Literal[3]
    metadata: TimeoutAutoModerationActionMetadataData


AutoModerationActionData = Union[
    BlockMessageAutoModerationActionData, SendAlertMessageAutoModerationActionData,
    TimeoutAutoModerationActionData
]


# https://discord.com/developers/docs/resources/auto-moderation#auto-moderation-action-object-action-types


AutoModerationActionTypes = Literal[1, 2, 3]


# https://discord.com/developers/docs/resources/auto-moderation#auto-moderation-action-object-action-metadata


@final
class SendAlertMessageAutoModerationActionMetadataData(TypedDict):
    channel_id: Snowflake


@final
class TimeoutAutoModerationActionMetadataData(TypedDict):
    duration_seconds: int


AutoModerationActionMetadataData = Union[
    SendAlertMessageAutoModerationActionMetadataData, TimeoutAutoModerationActionMetadataData
]



from __future__ import annotations

from typing import TYPE_CHECKING, Dict, List, Union

from typing_extensions import Literal, NotRequired, TypedDict, final

if TYPE_CHECKING:
    from ..reference import Snowflake
    from ..resources import (
        AllowedMentionsData, AttachmentData, EmbedData, GuildMemberData,
        MessageData, PartialChannelData, RoleData, UserData
    )
    from .commands import (
        ApplicationCommandOptionInteractionData, AutocompleteOptionData,
        SubcommandGroupOptionInteractionData, SubcommandOptionInteractionData
    )
    from .components import (
        ComponentData, SelectMenuComponentData, SelectMenuOptionData,
        TextInputComponentData
    )

__all__ = (
    'InteractionData', 'InteractionType', 'ResolvedInteractionDataData',
    'MessageInteractionData', 'InteractionResponseData', 'InteractionCallbackTypes',
    'InteractionCallbackData', 'ApplicationCommandInteractionData',
    'ComponentInteractionData', 'AutocompleteInteractionData', 'InteractionData',
    'InteractionMessageCallbackData', 'InteractionAutocompleteCallbackData',
    'InteractionModalCallbackData', 'InteractionMessageResponseData',
    'InteractionAutocompleteResponseData', 'InteractionModalResponseData',
    'InteractionNodataResponseData', 'ModalInteractionData'
)


# https://discord.com/developers/docs/interactions/receiving-and-responding#interaction-object-interaction-structure


class GuildInteractionData(TypedDict):
    id: Snowflake
    application_id: Snowflake
    guild_id: Snowflake
    channel_id: Snowflake
    member: GuildMemberData
    token: str
    version: int
    locale: str
    guild_locale: str


@final
class ApplicationCommandGuildInteractionData(GuildInteractionData):
    type: Literal[2]
    data: ApplicationCommandInteractionDataData
    app_permissions: str


@final
class ComponentGuildInteractionData(GuildInteractionData):
    type: Literal[3]
    data: ComponentInteractionDataData
    message: MessageData
    app_permissions: str


@final
class ModalGuildInteractionData(GuildInteractionData):
    type: Literal[5]
    data: ModalComponentInteractionDataData
    message: MessageData
    app_permissions: str


@final
class AutocompleteGuildInteractionData(GuildInteractionData):
    type: Literal[4]
    data: ApplicationCommandInteractionDataData


class ChannelInteractionData(TypedDict):
    id: Snowflake
    application_id: Snowflake
    channel_id: Snowflake
    user: UserData
    token: str
    version: int
    locale: str


@final
class ApplicationCommandChannelInteractionData(ChannelInteractionData):
    type: Literal[2]
    data: ApplicationCommandInteractionDataData
    app_permissions: str


@final
class ComponentChannelInteractionData(ChannelInteractionData):
    type: Literal[3]
    data: ComponentInteractionDataData
    message: MessageData
    app_permissions: str


@final
class ModalChannelInteractionData(ChannelInteractionData):
    type: Literal[5]
    data: ModalComponentInteractionDataData
    message: MessageData
    app_permissions: str


@final
class AutocompleteChannelInteractionData(ChannelInteractionData):
    type: Literal[4]
    data: ApplicationCommandInteractionDataData


class UserCommandInteractionBase(TypedDict):
    id: Snowflake
    application_id: Snowflake
    type: Literal[2]
    data: ContextMenuInteractionDataData
    token: str
    version: int
    locale: str
    app_permissions: str


@final
class GuildUserCommandInteractionData(UserCommandInteractionBase):
    member: GuildMemberData
    guild_locale: str


@final
class ChannelUserCommandInteractionData(UserCommandInteractionBase):
    user: UserData


ApplicationCommandInteractionData = Union[
    ApplicationCommandGuildInteractionData, ApplicationCommandChannelInteractionData,
    GuildUserCommandInteractionData, ChannelUserCommandInteractionData
]


ComponentInteractionData = Union[
    ComponentGuildInteractionData, ComponentChannelInteractionData
]


ModalInteractionData = Union[
    ModalGuildInteractionData, ModalChannelInteractionData
]

AutocompleteInteractionData = Union[
    AutocompleteGuildInteractionData, AutocompleteChannelInteractionData
]


InteractionData = Union[
    ApplicationCommandInteractionData, ComponentInteractionData, AutocompleteInteractionData,
    ModalInteractionData,
]


# https://discord.com/developers/docs/interactions/receiving-and-responding#interaction-object-interaction-type


InteractionType = Literal[1, 2, 3, 4, 5]


# https://discord.com/developers/docs/interactions/receiving-and-responding#interaction-object-interaction-data-structure


class ApplicationCommandInteractionDataBase(TypedDict):
    id: Snowflake
    name: str
    type: Literal[1, 2, 3]
    resolved: NotRequired[ResolvedInteractionDataData]
    options: NotRequired[
        List[Union[
            SubcommandOptionInteractionData,
            SubcommandGroupOptionInteractionData,
            ApplicationCommandOptionInteractionData
        ]]
    ]


@final
class ApplicationCommandInteractionDataData(ApplicationCommandInteractionDataBase):
    pass


@final
class ButtonComponentInteractionDataData(TypedDict):
    custom_id: str
    component_type: Literal[2]


@final
class SelectComponentInteractionDataData(TypedDict):
    custom_id: str
    component_type: Literal[3]
    values: List[SelectMenuOptionData]


@final
class ModalComponentInteractionDataData(TypedDict):
    custom_id: str
    component_type: Literal[4]
    components: List[Union[TextInputComponentData, SelectMenuComponentData]]


ComponentInteractionDataData = Union[
    ButtonComponentInteractionDataData, SelectComponentInteractionDataData,
    ModalComponentInteractionDataData
]


@final
class ContextMenuInteractionDataData(ApplicationCommandInteractionDataBase):
    target_id: Snowflake


InteractionDataData = Union[
    ApplicationCommandInteractionDataData, ButtonComponentInteractionDataData,
    SelectComponentInteractionDataData, ContextMenuInteractionDataData
]


# https://discord.com/developers/docs/interactions/receiving-and-responding#interaction-object-resolved-data-structure


@final
class ResolvedInteractionDataData(TypedDict):
    users: NotRequired[Dict[Snowflake, UserData]]
    members: NotRequired[Dict[Snowflake, GuildMemberData]]
    roles: NotRequired[Dict[Snowflake, RoleData]]
    channels: NotRequired[Dict[Snowflake, PartialChannelData]]
    messages: NotRequired[Dict[Snowflake, MessageData]]


# https://discord.com/developers/docs/interactions/receiving-and-responding#message-interaction-object-message-interaction-structure


@final
class MessageInteractionData(TypedDict):
    id: Snowflake
    type: Literal[2, 3, 4]
    name: str
    user: UserData


# https://discord.com/developers/docs/interactions/receiving-and-responding#interaction-response-object-interaction-response-structure


@final
class InteractionMessageResponseData(TypedDict):
    type: Literal[4, 7]
    data: InteractionMessageCallbackData


@final
class InteractionAutocompleteResponseData(TypedDict):
    type: Literal[8]
    data: InteractionAutocompleteCallbackData


@final
class InteractionModalResponseData(TypedDict):
    type: Literal[9]
    data: InteractionModalCallbackData


@final
class InteractionNodataResponseData(TypedDict):
    type: Literal[1, 5, 6]


InteractionResponseData = Union[
    InteractionMessageResponseData, InteractionAutocompleteResponseData,
    InteractionModalResponseData, InteractionNodataResponseData
]


# https://discord.com/developers/docs/interactions/receiving-and-responding#interaction-response-object-interaction-callback-type


InteractionCallbackTypes = Literal[1, 4, 5, 6, 7, 8, 9]


# https://discord.com/developers/docs/interactions/receiving-and-responding#interaction-response-object-interaction-callback-data-structure


@final
class InteractionMessageCallbackData(TypedDict):
    tts: NotRequired[bool]
    content: NotRequired[str]
    embeds: NotRequired[List[EmbedData]]
    allowed_mentions: NotRequired[AllowedMentionsData]
    flags: NotRequired[int]
    components: NotRequired[List[ComponentData]]
    attachments: NotRequired[List[AttachmentData]]


@final
class InteractionAutocompleteCallbackData(TypedDict):
    choices: List[AutocompleteOptionData]


@final
class InteractionModalCallbackData(TypedDict):
    custom_id: str
    title: str
    components: List[ComponentData]


InteractionCallbackData = Union[
    InteractionMessageCallbackData, InteractionAutocompleteCallbackData,
    InteractionModalCallbackData
]



from __future__ import annotations

from typing import TYPE_CHECKING

from typing_extensions import Literal, TypedDict, final

if TYPE_CHECKING:
    from ..reference import Snowflake

__all__ = ('StageInstanceData', 'StageInstancePrivacyLevels')


# https://discord.com/developers/docs/resources/stage-instance#stage-instance-object-stage-instance-structure


@final
class StageInstanceData(TypedDict):
    id: Snowflake
    guild_id: Snowflake
    channel_id: Snowflake
    topic: str
    privacy_level: StageInstancePrivacyLevels
    discoverable_disabled: bool


# https://discord.com/developers/docs/resources/stage-instance#stage-instance-object-privacy-level


StageInstancePrivacyLevels = Literal[1, 2]



from __future__ import annotations

from typing import TYPE_CHECKING, Dict, List, Optional, Union

from typing_extensions import Literal, NotRequired, TypedDict, final

if TYPE_CHECKING:
    from ..reference import Snowflake

__all__ = (
    'ApplicationCommandData', 'ApplicationCommandTypes', 'SubcommandOptionData',
    'SubcommandGroupOptionData', 'AutocompleteOptionData', 'ApplicationCommandOptionData',
    'ApplicationCommandOptionInteractionData', 'GuildApplicationCommandPermissionData',
    'ApplicationCommandPermissionsData', 'ApplicationCommandPayload',
    'BatchEditApplicationCommandPermissionsData'
)


# https://discord.com/developers/docs/reference#locales


Locales = Literal[
    'da', 'de', 'en-GB', 'en-US', 'en-ES', 'fr', 'hr', 'it', 'lt', 'hu', 'nl',
    'no', 'pl', 'pt-BR', 'ro', 'fi', 'sv-SE', 'vi', 'tr', 'cs', 'el', 'bg',
    'ru', 'uk', 'hi', 'th', 'zh-CN', 'ja', 'zh-TW', 'ko'
]


# https://discord.com/developers/docs/interactions/application-commands#application-command-object-application-command-structure


@final
class ChatInputCommandData(TypedDict):
    id: Snowflake
    type: NotRequired[Literal[1]]
    application_id: Snowflake
    guild_id: NotRequired[Snowflake]
    name: str
    name_localizations: NotRequired[Optional[Dict[Locales, str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict[Locales, str]]]
    options: List[ApplicationCommandOptionData]
    default_permission: NotRequired[bool]
    version: Snowflake


@final
class ContextMenuCommandData(TypedDict):
    id: Snowflake
    type: NotRequired[Literal[2, 3]]
    application_id: Snowflake
    guild_id: NotRequired[Snowflake]
    name: str
    name_localizations: NotRequired[Optional[Dict[Locales, str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict[Locales, str]]]
    default_permission: NotRequired[bool]
    version: Snowflake


ApplicationCommandData = Union[ChatInputCommandData, ContextMenuCommandData]


# https://discord.com/developers/docs/interactions/application-commands#application-command-object-application-command-types


ApplicationCommandTypes = Literal[1, 2, 3]


# https://discord.com/developers/docs/interactions/application-commands#application-command-object-application-command-option-structure
# The variations and overloads of options are huge, but we want to provide
# extremely accurate typing - this is gonna be a huge Union with a lot of
# TypedDict subclasses...


@final
class SubcommandOptionData(TypedDict):
    type: Literal[1]
    name: str
    name_localizations: NotRequired[Optional[Dict[Locales, str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict[Locales, str]]]
    options: NotRequired[List[ApplicationCommandOptionData]]


@final
class SubcommandGroupOptionData(TypedDict):
    type: Literal[2]
    name: str
    name_localizations: NotRequired[Optional[Dict[Locales, str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict[Locales, str]]]
    options: List[SubcommandOptionData]


@final
class ChoicesStringOptionData(TypedDict):
    type: Literal[3]
    name: str
    name_localizations: NotRequired[Optional[Dict[Locales, str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict[Locales, str]]]
    required: NotRequired[bool]
    choices: NotRequired[List[StrCommandOptionChoiceData]]


@final
class AutocompleteStringOptionData(TypedDict):
    type: Literal[3]
    name: str
    name_localizations: NotRequired[Optional[Dict[Locales, str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict[Locales, str]]]
    required: NotRequired[bool]
    autocomplete: NotRequired[bool]


@final
class MinMaxStringOptionData(TypedDict):
    type: Literal[3]
    name: str
    name_localizations: NotRequired[Optional[Dict[Locales, str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict[Locales, str]]]
    required: NotRequired[bool]
    min_length: NotRequired[int]
    max_length: NotRequired[int]


@final
class ChoicesIntegerCommandOptionData(TypedDict):
    type: Literal[4]
    name: str
    name_localizations: NotRequired[Optional[Dict[Locales, str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict[Locales, str]]]
    required: NotRequired[bool]
    choices: NotRequired[List[IntCommandOptionChoiceData]]


@final
class MinMaxIntegerCommandOptionData(TypedDict):
    type: Literal[4]
    name: str
    name_localizations: NotRequired[Optional[Dict[Locales, str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict[Locales, str]]]
    required: NotRequired[bool]
    min_value: NotRequired[int]
    max_value: NotRequired[int]


@final
class AutocompleteIntegerOptionData(TypedDict):
    type: Literal[4]
    name: str
    name_localizations: NotRequired[Optional[Dict[Locales, str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict[Locales, str]]]
    required: NotRequired[bool]
    autocomplete: NotRequired[bool]


@final
class BooleanOptionData(TypedDict):
    type: Literal[5]
    name: str
    name_localizations: NotRequired[Optional[Dict[Locales, str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict[Locales, str]]]
    required: NotRequired[bool]


@final
class UserOptionData(TypedDict):
    type: Literal[6]
    name: str
    name_localizations: NotRequired[Optional[Dict[Locales, str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict[Locales, str]]]
    required: NotRequired[bool]


@final
class ChannelOptionData(TypedDict):
    type: Literal[7]
    name: str
    name_localizations: NotRequired[Optional[Dict[Locales, str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict[Locales, str]]]
    required: NotRequired[bool]
    channel_types: NotRequired[List[Literal[0, 2, 4, 5, 6, 13]]]


@final
class RoleOptionData(TypedDict):
    type: Literal[8]
    name: str
    name_localizations: NotRequired[Optional[Dict[Locales, str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict[Locales, str]]]
    required: NotRequired[bool]


@final
class MentionableOptionData(TypedDict):
    type: Literal[9]
    name: str
    name_localizations: NotRequired[Optional[Dict[Locales, str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict[Locales, str]]]
    required: NotRequired[bool]


@final
class ChoicesNumberCommandOptionData(TypedDict):
    type: Literal[10]
    name: str
    name_localizations: NotRequired[Optional[Dict[Locales, str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict[Locales, str]]]
    required: NotRequired[bool]
    choices: NotRequired[List[NumberCommandOptionChoiceData]]


@final
class MinMaxNumberCommandOptionData(TypedDict):
    type: Literal[10]
    name: str
    name_localizations: NotRequired[Optional[Dict[Locales, str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict[Locales, str]]]
    required: NotRequired[bool]
    min_value: NotRequired[Union[int, float]]
    max_value: NotRequired[Union[int, float]]


@final
class AutocompleteNumberOptionData(TypedDict):
    type: Literal[10]
    name: str
    name_localizations: NotRequired[Optional[Dict[Locales, str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict[Locales, str]]]
    required: NotRequired[bool]
    autocomplete: NotRequired[bool]


@final
class AttachmentOptionData(TypedDict):
    type: Literal[10]
    name: str
    name_localizations: NotRequired[Optional[Dict[Locales, str]]]
    description: str
    description_localizations: NotRequired[Optional[Dict[Locales, str]]]
    required: NotRequired[bool]


AutocompleteOptionData = Union[
    AutocompleteStringOptionData, AutocompleteIntegerOptionData,
    AutocompleteNumberOptionData
]


ApplicationCommandOptionData = Union[
    ChoicesStringOptionData, ChoicesIntegerCommandOptionData,
    MinMaxIntegerCommandOptionData, MinMaxStringOptionData, BooleanOptionData,
    UserOptionData, ChannelOptionData, RoleOptionData, MentionableOptionData,
    ChoicesNumberCommandOptionData, MinMaxNumberCommandOptionData,
    AutocompleteOptionData, SubcommandOptionData, AttachmentOptionData,
]


# https://discord.com/developers/docs/interactions/application-commands#application-command-object-application-command-option-type


ApplicationCommandOptionTypes = Literal[1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]


# https://discord.com/developers/docs/interactions/application-commands#application-command-object-application-command-option-choice-structure


@final
class StrCommandOptionChoiceData(TypedDict):
    name: str
    name_localizations: NotRequired[Optional[Dict[Locales, str]]]
    value: str


@final
class IntCommandOptionChoiceData(TypedDict):
    name: str
    name_localizations: NotRequired[Optional[Dict[Locales, str]]]
    value: int


@final
class NumberCommandOptionChoiceData(TypedDict):
    name: str
    name_localizations: NotRequired[Optional[Dict[Locales, str]]]
    value: Union[int, float]


# https://discord.com/developers/docs/interactions/application-commands#application-command-object-application-command-interaction-data-option-structure


@final
class SubcommandOptionInteractionData(TypedDict):
    name: str
    type: Literal[1]
    options: List[ApplicationCommandOptionInteractionData]


@final
class SubcommandGroupOptionInteractionData(TypedDict):
    name: str
    type: Literal[2]
    options: List[SubcommandOptionInteractionData]


@final
class StringOptionInteractionData(TypedDict):
    name: str
    type: Literal[3]
    value: str
    focused: NotRequired[bool]


@final
class IntegerOptionInteractionData(TypedDict):
    name: str
    type: Literal[4]
    value: int
    focused: NotRequired[bool]


@final
class BooleanOptionInteractionData(TypedDict):
    name: str
    type: Literal[5]
    value: bool


@final
class UserOptionInteractionData(TypedDict):
    name: str
    type: Literal[6]
    value: Snowflake


@final
class ChannelOptionInteractionData(TypedDict):
    name: str
    type: Literal[7]
    value: Snowflake


@final
class RoleOptionInteractionData(TypedDict):
    name: str
    type: Literal[8]
    value: Snowflake


@final
class MentionableInteractionData(TypedDict):
    name: str
    type: Literal[9]
    value: Snowflake


@final
class NumberInteractionData(TypedDict):
    name: str
    type: Literal[10]
    value: Union[int, float]
    focused: NotRequired[bool]


@final
class AttachmentInteractionData(TypedDict):
    name: str
    type: Literal[10]
    value: str


ApplicationCommandOptionInteractionData = Union[
    SubcommandOptionInteractionData, SubcommandGroupOptionInteractionData,
    StringOptionInteractionData, IntegerOptionInteractionData, BooleanOptionInteractionData,
    UserOptionInteractionData, ChannelOptionInteractionData, RoleOptionInteractionData,
    MentionableInteractionData, NumberInteractionData, AttachmentInteractionData,
]


# https://discord.com/developers/docs/interactions/application-commands#application-command-permissions-object-guild-application-command-permissions-structure


@final
class GuildApplicationCommandPermissionData(TypedDict):
    id: Snowflake
    application_id: Snowflake
    guild_id: Snowflake
    permissions: ApplicationCommandPermissionsData


# https://discord.com/developers/docs/interactions/application-commands#application-command-permissions-object-application-command-permissions-structure


@final
class ApplicationCommandPermissionsData(TypedDict):
    id: Snowflake
    type: ApplicationCommandPermissionTypes
    permission: bool


# https://discord.com/developers/docs/interactions/application-commands#application-command-permissions-object-application-command-permission-type


ApplicationCommandPermissionTypes = Literal[1, 2, 3]


# https://discord.com/developers/docs/interactions/application-commands#create-global-application-command-json-params


@final
class ApplicationCommandPayload(TypedDict):
    name: str
    description: str
    options: NotRequired[List[ApplicationCommandOptionData]]
    default_permission: NotRequired[bool]
    type: NotRequired[Literal[1, 2, 3]]


# https://discord.com/developers/docs/interactions/application-commands#batch-edit-application-command-permissions-example


@final
class BatchEditApplicationCommandPermissionsData(TypedDict):
    id: Snowflake
    permissions: ApplicationCommandPermissionsData
